import Type from "./type";
import Launch from "../utils/launch";
import Action from "../action";

let loading = false;

export default class {
  static _extractFormData(properties, needCsrfToken) {
    if (properties["formDataEncoded"]) {
      return properties["formData"];
    }

    const formData = new FormData();
    var params = properties["formData"] || {};
    for (const key in params) {
      formData.append(key, params[key]);
    }

    if (needCsrfToken && !Type.isString(params["authenticity_token"])) {
      formData.append("authenticity_token", Utils.dom.getCsrf());
      // formData.append("authenticity_token", Storage.getLocal(Keys.Db.csrfToken))
    }
    return formData;
  }

  static load(properties, target, component) {
    if (Utils.settings.reactive) {
      const currentUrl = window.location.href;
      const htmlUrl = Utils.url.htmlUrl(properties["url"]);

      Utils.http.execute(properties, "GET", component, (data, response) => {
        if (htmlUrl !== currentUrl) {
          const redirectUrl = Utils.url.htmlUrl(response.url);
          Utils.history.pushPage(data, redirectUrl);
          Utils.history.restoreOnBackOrForward();
        }

        this.forceComponentUpdate(() => {
          window.vueApp.page = data;
          Action.execute(properties["onOpen"], target, component);
        });
      });

      // // TODO: Consider moving the condition to inside Utils.http.execute()'s block
      // // Avoid inserting duplicate entries into history
      // if (htmlUrl !== currentUrl) {
      //   Utils.http.execute(properties, "GET", component, (data, response) => {
      //     const redirectUrl = Utils.url.htmlUrl(response.url);
      //     window.vueApp.page = data;
      //     Utils.history.pushPage(data, redirectUrl);
      //     Utils.history.restoreOnBackOrForward();
      //     Action.execute(properties["onOpen"], target, component);
      //   });
      // } else {
      //   // console.warn("Refuse opening the same URL");
      //   Utils.http.execute(properties, "GET", component, (data, response) => {
      //     // const redirectUrl = Utils.url.htmlUrl(response.url);
      //     window.vueApp.page = data;
      //     // Utils.history.pushPage(data, redirectUrl);
      //     // Utils.history.restoreOnBackOrForward();
      //     Action.execute(properties["onOpen"], target, component);
      //   });
      // }
    } else {
      window.location = Utils.url.htmlUrl(properties["url"]);
    }
  }

  static execute(properties, methodName, component, jsonHandler) {
    this.startIndicator(component);

    // `fetch()` only supports uppercase
    const method = methodName.toUpperCase();
    let url = Utils.url.jsonUrl(properties["url"]);
    let body = this._extractFormData(properties, method !== "GET");
    if (method === "GET") {
      url = Utils.url.appendParams(url, body);
      body = null;
    }
    console.log(`${method} ${url}`);

    let response = null;
    const vm = this;
    fetch(url, {
      method: method,
      body: body,
      headers: { Accept: "text/html" },
      // Make sure to pass cookies for same origin URLs.
      // Needed for some browsers, e.g. Edge and Android's native.
      credentials: "same-origin"
    })
      .then(res => {
        vm.stopIndicator(component);

        if (res.status >= 500) {
          throw 'Server error'
        } else {
          response = res;
          return res.json();
        }
      })
      .then(data => {
        console.debug("Success", data);
        jsonHandler(data, response);
      })
      .catch(error => {
        console.error("Error:", error);
        vm.stopIndicator(component);
        Launch.snackbar({ message: error });
      });
  }

  static startIndicator(component) {
    this._showIndicator();
    component.$data._isBusy = true;
  }

  static stopIndicator(component) {
    this._hideIndicator();
    component.$data._isBusy = false;
  }

  static _showIndicator() {
    loading = true;
    setTimeout(() => {
      if (loading) {
        loading = false;
        window.vueApp.indicator = true;
      }
    }, 200);
  }

  static _hideIndicator() {
    loading = false;
    window.vueApp.indicator = false;
  }

  // See generic.js
  static forceComponentUpdate(handler) {
    window.vueApp.isStale = true;

    // Queue the execution so the first isStale has time to resets state before handler gets executed
    setTimeout(() => {
      handler();
      window.vueApp.isStale = false;
    }, 0);
  }
}
